<?php
/**
 * @file
 * Page callbacks for Garmin Connector.
 */

/**
 * Page callback: Show a demo page.
 *
 * @return array
 *   A drupal render array
 *
 * @see garmin_connector_menu()
 */
function garmin_connector_show_demo() {
  module_load_include('inc', 'garmin_connector', 'garmin_connector.catalogue');
  $overview = "<h3>" . t("Welcome to Garmin Connector") . "</h3>";
  $overview .= garmin_connector_get_availability();
  $overview .= garmin_connector_get_design_rationale();
  $overview .= "<h3>" . t("Technical Considerations") . "</h3>";
  $overview .= "<p>";
  $overview .= t("Garmin provides a middleware to !garminconnect. This middleware contains the following characteristics",
    array(
      '!garminconnect'  => "<a href='http://connect.garmin.com'>" .
      t("Garmin Connect") . "</a>",
    )
  ) . "</p>";
  $overview .= "<ol>";
  $overview .= "<li>" . t("Zero support") . "</li>";
  $overview .= "<li>" . t("No reference implementation") . "</li>";
  $overview .= "<li>" . t("Partial documentation that is often incorrect") . "</li>";
  $overview .= "</ol>";
  $overview .= "<p>" . t("The documentation for the API is found here:") .  "</p>";
  $overview .= "<ol>";
  $overview .= "<li>" . t("!ass",
    array(
      '!ass'  => "<a href='http://connect.garmin.com/proxy/activity-search-service-1.2/rest.activities.html'>" . t("Activity Search Service") . "</a>",
    )
  ) . "</li>";
  $overview .= "<li>" . t("!as",
    array(
      '!as'  => "<a href='http://connect.garmin.com/proxy/activity-service-1.2/rest.activities.html'>" . t("Activity Service") . "</a>",
    )
  ) . "</li>";
  $overview .= "</ol>";
  $overview .= "<p>" . t("I attempted the project because it was the most appealing of the distasteful options.") . "</p>";
  $overview .= "<h3>" . t("Getting Started") . "</h3>";
  $overview .= "<p>" . t("The first thing that you should so is configure Garmin Connector !here",
    array(
      '!here' => "<a href='" .  url('admin/config/garmin_connector/garmin_connector') . "'>" . t("here") .  "</a>",
    )
  );
  $overview .= "</p>";
  $overview .= "<p>" . t("Notice the blocks on this page?") . "</p>\n";
  $overview .= "<ol><li>" . t("The thick red-bordered block is a list of the last ten running workouts") . "</li>";
  $overview .= "<li>" . t("The thick green-bordered block is a list of the last ten biking workouts") . "</li>";
  $overview .= "</ol>";
  $overview .= "<p>";
  $overview .= t("Notice the presence of the block with the pink border? This is the Quick View Pane. It shows you a graph of your activity when you press the 'Quick View' link near an activity");
  $overview .= "</p>";
  $overview .= "<p>" . t("Here are some other pages in the Garmin Connector system:") . "</p>\n";
  $overview .= "<ol>\n";
  $overview .= "<li>";
  $overview .= garmin_connector_get_demo_link('view activities', 'garmin-connector/last-ten-hard/run.html', t("Your last ten hard run workouts"));
  $overview .= "</li>";
  $overview .= "<li>";
  $overview .= garmin_connector_get_demo_link('view activities', 'garmin-connector/last-ten-hard/bike.html', t("Your last ten hard bike workouts"));
  $overview .= "</li>";
  $overview .= "<li>";
  $overview .= garmin_connector_get_demo_link('manage activities', 'garmin-connector/activity-manager.html', t("The Garmin Connector Activity Manager"));
  $overview .= "<p>" . t("Manage activity files on the filesystem and in the Garmin Connector Activity Catalogue.") . "</p>";
  $overview .= "</li>";
  $overview .= "<li>";
  $overview .= garmin_connector_get_demo_link('manage activities', 'garmin-connector/purge.html', t("The Garmin Connector Activity Purger"));
  $overview .= "<p>" . t("Remove activities from the filesystem and the Garmin Connector Activity Catalogue.") . "</p>";
  $overview .= "</li>";
  $overview .= "<li>";
  $overview .= garmin_connector_get_demo_link('view activities', 'garmin-connector/compare.html', t("The Garmin Connector Activity Comparator"));
  $overview .= "<p>" . t("This is a non-persistent way to compare workouts. Great for spur of the moment comparisons. If you want the comparisons to be retained, use an activity set.") . "</p>";
  $overview .= "</li>";
  $overview .= "<li>";
  $overview .= garmin_connector_get_demo_link('view activity sets', "garmin-connector/activity-sets.html", t("Activity Sets"));
  $overview .= "<p>" . t("This is a persistent way to compare workouts. Great for preparing lists to show your coach. If you want quick, non-persistent comparisons use the Activity Comparator.") . "</p>";
  $overview .= "</li>";
  $overview .= "</ol>";
  $activities = garmin_connector_get_activities_from_local();
  $returner = array();
  $returner['overview'] = array(
    '#markup' => $overview);
  $returner['runs']['#prefix'] = "<h4>" . t("Last ten run activities") . "</h4>";
  $returner['runs']  = garmin_connector_get_last_ten_activities($activities, "running");
  $returner['runs']['#theme'] = 'garmin_connector_block_wrapper';
  $returner['bikes'] = garmin_connector_get_last_ten_activities($activities, "cycling");
  $returner['bikes']['#theme'] = 'garmin_connector_block_wrapper';
  $returner['viewer'] = garmin_connector_block_view('quick-view-result-pane');
  $returner['#prefix'] = "<div class='garmin-connector-demo'>";
  $returner['#suffix'] = "</div><!-- /gc-demo -->";
  return $returner;
}

/**
 * Page callback: Constructs a form for administering Garmin Connector.
 *
 * @see garmin_connector_menu()
 */
function garmin_connector_admin() {
  $form = array();
  $form['#attached']['css'] = array(
    drupal_get_path('module', 'garmin_connector') . '/garmin_connector.css',
  );
  $form['garmin_connector_username'] = array(
    '#type' => 'textfield',
    '#title' => t('Garmin Connect Username'),
    '#size' => 20,
    '#maxlength' => 20,
    '#default_value' => variable_get('garmin_connector_username'),
    '#description' => t("Your Garmin Connect username."),
    '#required' => TRUE,
  );
  $oldpw = variable_get('garmin_connector_pw');
  $form['garmin_connector_pw'] = array(
    '#type' => 'password_confirm',
    '#title' => t('Garmin Connect Password'),
    '#size' => 20,
    '#description' => (
      t("Your Garmin Connect password (feel free to ignore the advice on a strong password. It is currently set to (@pw) You can tell Garmin Connector your current Garmin Connect here.",
        array(
          '@pw' => $oldpw,
        )
      )),
    '#required' => (strlen($oldpw) == 0) ? TRUE : FALSE,
  );
  $form['garmin_connector_max_heart_rate_running'] = array(
    '#type' => 'textfield',
    '#title' => t('Max Heart Rate (running)'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => variable_get('garmin_connector_max_heart_rate_running'),
    '#description' => t("Your max heart rate while running. Used to draw graphs."),
    '#required' => FALSE,
  );
  $form['garmin_connector_z2_heart_rate_running'] = array(
    '#type' => 'textfield',
    '#title' => t('Zone 2 Heart Rate (running)'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => variable_get('garmin_connector_z2_heart_rate_running'),
    '#description' => t("Your zone 2 heart rate while running. Used to draw graphs."),
    '#required' => FALSE,
  );
  $form['garmin_connector_max_heart_rate_cycling'] = array(
    '#type' => 'textfield',
    '#title' => t('Max Heart Rate (cycling)'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => variable_get('garmin_connector_max_heart_rate_cycling'),
    '#description' => t("Your max heart rate while cycling. Used to draw graphs."),
    '#required' => FALSE,
  );
  $form['garmin_connector_z2_heart_rate_cycling'] = array(
    '#type' => 'textfield',
    '#title' => t('Zone 2 Heart Rate (cycling)'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => variable_get('garmin_connector_z2_heart_rate_cycling'),
    '#description' => t("Your Zone 2 heart rate while cycling. Used to draw graphs."),
    '#required' => FALSE,
  );
  $form['garmin_connector_username'] = array(
    '#type' => 'textfield',
    '#title' => t('Garmin Connect Username'),
    '#size' => 20,
    '#maxlength' => 20,
    '#default_value' => variable_get('garmin_connector_username'),
    '#description' => t("Your Garmin Connect username."),
    '#required' => TRUE,
  );
  $form['garmin_connector_key_workout_block_ignore_regex'] = array(
    '#type' => 'textfield',
    '#title' => t('Key Workout Block Ignore Regular Expression'),
    '#size' => 100,
    '#maxlength' => 100,
    '#default_value' => variable_get(
      'garmin_connector_key_workout_block_ignore_regex',
      "Zone 2|Z2|Recovery|Warmup"),
    '#description' => t("Garmin Connector has a block called 'Key Workouts'. For me, that's any workout that is not zone 2. Garmin Connector needs to know how to distinguish your key workouts from your other ones. Some people put the string 'Z2' or 'Zone 2' in the description for the workout. This string is a perl regular expression like 'Zone 2|Zone2|Z2|Recovery|Warmup'. When a workout's description matches this, it is left out of the 'Hard Workout' block."),
    '#required' => TRUE,
  );
  $form['garmin_connector_reward_regex'] = array(
    '#type' => 'textfield',
    '#title' => t('Reward Block Regex'),
    '#size' => 100,
    '#maxlength' => 100,
    '#default_value' => variable_get(
      'garmin_connector_reward_regex',
      "Corrective Exercises|Core"),
    '#description' => t("Sometimes doing distasteful workouts is hard. For Chad, that's his assigned core exercises. Garmin Connector has a block that will shout out to you when you've completed certain workouts. This regular expression selects workouts to evaluate as described below."),
    '#required' => TRUE,
  );
  $form['garmin_connector_reward_number_of_days'] = array(
    '#type' => 'textfield',
    '#title' => t('Reward Days Back'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => variable_get(
      'garmin_connector_reward_number_of_days',
      "7"),
    '#description' => t("The number of days back from today to scan for reward workouts."),
    '#required' => TRUE,
  );
  $form['garmin_connector_reward_green_days'] = array(
    '#type' => 'textfield',
    '#title' => t('Reward Green Days'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => variable_get(
      'garmin_connector_reward_green_days',
      "6"),
    '#description' => t("If you did this many reward workouts you get a green!"),
    '#required' => TRUE,
  );
  $form['garmin_connector_reward_yellow_days'] = array(
    '#type' => 'textfield',
    '#title' => t('Reward Yellow Days'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => variable_get(
      'garmin_connector_reward_yellow_days', "5"),
    '#description' => t("If you did this many reward workouts you get a yellow. Less than this many you get a red."),
    '#required' => TRUE,
  );
  global $base_url;
  $cwd = getcwd();
  $drusher = "<code>drush -r $cwd -l $base_url help</code>";
  // Avoid the strict Warning by getting the form first.
  $form['run_retrieve_activities_now'] = array(
    '#markup' => "<p>" .
    t("After you've set your Garmin Connect username and password, run Garmin Connector retrieve activities !here to download activities from Garmin Connect. Garmin Connector also has a collection of drush methods for being more specific with downloading and refreshing activities from Garmin Connnect. Run !drush on your commandline to get a list!",
      array(
        '!here' => "<a href='" .
        url("garmin-connector/activity-manager.html") . "'>" . t("here") . "</a>",
        '!drush' => $drusher,
      )
    ) . "</p>");
  $maintenance = t("Activity Manager");
  $message = "<p>" .
    t("Garmin Connector has an !catalogue_maintenance_link that lists important URLs for helping you maintain your catalogue of activities. Visit it soon!",
    array(
      '!catalogue_maintenance_link' =>
      "<a href='" . url('garmin-connector/activity-manager.html') .
      "'>" . $maintenance . "</a>",
    )
  ) . "</p>";
  $form['additional-help'] = array(
    '#markup' => $message,
  );
  $commander = garmin_connector_get_drush_commands();
  $form['drush'] = $commander;
  $form['drush']['#prefix']  = "<div>";
  $form['drush']['#prefix'] .= t("Garmin Connector has a collection of drush methods for doing all sorts of things. Run !drush on the commandline to get a list of drush methods. Here's a brief list:",
    array(
      '!drush' => $drusher,
    ));
  $form['drush']['#suffix'] = "</div>";
  return system_settings_form($form);
}

/**
 * Page callback: Provide a gui for maintaining activities in Garmin Connector.
 *
 * This gui will allow you to add to workout sets, purge, run retrieve 
 * activities, whatever.
 *
 * @return array
 *   A drupal render array
 *
 * @see garmin_connector_menu()
 */
function garmin_connector_activity_manager() {
  module_load_include('inc', 'garmin_connector', 'garmin_connector.catalogue');
  garmin_connector_add_css_js();
  $workouts = garmin_connector_get_activities_from_local();
  $returner = array();
  $theform = drupal_get_form('garmin_connector_ajax_run_retrieve_activities_form');
  $header = drupal_render($theform) . "<div class='retrieve-activities-done-results'></div>";
  $header .= "<div class='workout-manager'>\n";
  $header .= "<div class='workout-manager header'>\n";
  $header .= "<a href='' onclick='toggle_workout_type_other(); ";
  $header .= "return false;'>";
  $header .= t("Toggle workouts of type other");
  $header .= "</a>";
  $header .= t("This is done to hide, for example, strength workouts");
  $header .= "</div><!-- /workout-manager header -->\n";
  $returner['#prefix'] = $header;
  foreach ($workouts as $workout) {
    $row = garmin_connector_get_activity_manager_row($workout);
    $returner[$workout->activityId] = $row;
  }
  $returner['#suffix'] = "</div><!-- /workout-manager -->\n";
  return $returner;
}

/**
 * Page callback: Get the last ten hard activities.
 *
 * @param string $type
 *   The type of workouts to get
 * 
 * @see garmin_connector_menu()
 */
function garmin_connector_last_ten_hard($type) {
  module_load_include('inc', 'garmin_connector', 'garmin_connector.catalogue');
  garmin_connector_add_css_js();
  $activities = garmin_connector_get_activities_from_local();
  if (user_access('view activities') == FALSE) {
    return array(
      '#markup' => "<div class='gc-no-permission'>" . t('Unfortunately, you do not have permission to view activity data. Ask your system administrator to help!') . "</div>",
    );
  }
  return garmin_connector_get_last_ten_activities($activities, $type, TRUE);
}

/**
 * Provides a block that rewards you when you do workouts that you hate.
 *
 * Example: core workouts
 * 
 * @return array
 *   A drupal render array
 */
function garmin_connector_get_reward_pane() {
  $workouts = garmin_connector_get_activities_from_local();
  $days = variable_get('garmin_connector_reward_number_of_days', 7);
  $cutoff = new DateTime("$days days ago");
  $green = variable_get('garmin_connector_reward_green_days', 6);
  $yellow = variable_get('garmin_connector_reward_yellow_days', 5);
  $regex = variable_get('garmin_connector_reward_regex',
  "Corrective Exercises|Core");
  $collected = array();
  foreach ($workouts as $workout) {
    $a = $workout;
    $name = $a->activityName;
    $time = new DateTime($a->activitySummary->BeginTimestamp->value);
    if (!preg_match("/$regex/", $name)) {
      continue;
    }
    if ($time < $cutoff) {
      continue;
    }
    $collected[$time->format('Y-m-d H:i:s')] = $a;
  }
  $s = count($collected);
  if ($s >= $green) {
    $color = "green";
  }
  elseif ($s >= $yellow) {
    $color = "yellow";
  }
  else {
    $color = "red";
  }
  $message = "<h2>" . t('Did you complete the workouts you hate?') . "</h2>";
  $message .= "<div class='reward $color'><p>";
  $message .= t("You completed @s reward workouts on @days days and earned @color!",
    array(
      '@s' => $s,
      '@days' => $days,
      '@color' => $color));
  $message .= "</p><p>";
  $message .= t("Your reward workouts are described using this expression: (@regex)",
    array('@regex' => $regex));
  $message .= "<p>"
    . t("You can change that expression in Garmin Connector's settings !here",
      array(
        '!here' => "<a href='" .
        url("admin/config/garmin_connector/garmin_connector") . "'>" .
        t("here") . "</a>",
      )
    )
  . "</p>";
  $message .= "</div>";
  return array(
    '#markup' => $message,
  );
}

/**
 * Provides a container that will be used to hold the GC quickview pane.
 *
 * @param int $count
 *   Sometimes there is more than quickview pane per page. This will allow you 
 *   to specify the id of the container
 *
 * @param int $id
 *   Include a javascript function to put data into the plot. The id is the
 *   activityId who's data will go into the plot.
 *
 * @return array
 *   a drupal render array
 */
function garmin_connector_get_plot_container($count = 0, $id = NULL) {
  flot_add_js('core');
  // This is needed to enable zoom functions.
  drupal_add_js(drupal_get_path('module', 'flot') .
  "/flot/jquery.flot.navigate.js");
  $container = "";
  $container .= "<div class='sport'></div>\n";
  $container .= "<div class='controls-container'><a href='' class='show-time-";
  $container .= "frame' onclick='garmin_connector_show_time(jQuery(this),\"all";
  $container .= "\"); return false;'>" . t("Zoom Out") . "</a>";
  $container .= "<a href='' class='show-time-frame' onclick='garmin_connector_";
  $container .= "show_time(jQuery(this),10); return false;'>";
  $container .= t("Zoom In") . "</a></div>\n";
  $container .= "<div class='plot-window'></div>\n";
  $container .= "<div class='activity-summary'></div>\n";
  $container .= "<div class='garmin-link'></div>\n";
  if ($id) {
    $uri = "public://garmin_connector/activities/plot-data/$id.json";
    $url = file_create_url($uri);
    $container .= "<script type='text/javascript'>garmin_connector_create_plot";
    $container .= "('$url','#plot-container-$count');</script>\n";
  }
  return array(
    '#prefix' => "<div id='plot-container-" . $count .
    "' class='plot-container'>\n",
    'title' => array(
      '#markup' => "<div class='title'>" . t("Activity Graph Quick View") .
      "</div>\n",
    ),
    'container' => array(
      '#markup' => $container,
    ),
    '#suffix' => "</div><!-- /plot-container -->\n",
  );
}


/**
 * Get HTML to provide links to download Garmin Connector.
 *
 * @return string
 *   HTML expressing download availability.
 */
function garmin_connector_get_availability() {
  $html = "";
  $html .= "<p>" . t("!gc is a full project with !drupal.",
    array(
      '!gc' => "<a href='http://drupal.org/project/garmin_connector'>" . t("Garmin Connector") . "</a>",
      '!drupal' => "<a href='http://drupal.org'>" . t("Drupal") . "</a>",
    )
  ) . "</p>";
  $html .= "<p>You can install Garmin Connector in the following ways:</p>";
  $html .= "<ul>";
  $html .= "<li>" . t("Using drush like this (recommended):");
  $html .= "<br /><code>$ drush -r your_drupal_root -l http://yourwebsite.com dl garmin_connector</code>";
  $html .= "<br /><code>$ drush -r your_drupal_root -l http://yourwebsite.com en garmin_connector</code>";
  $html .= "<p>" . t("Note that Garmin Connector provides numerous drush functions for operating Garmin Connector. Check them out!") . "</p>";
  $html .= "</li>";
  $html .= "<li>" . t("Manually by downloading it !here",
    array(
      "!here" => "<a href='http://drupal.org/projects/garmin_connector'>"
      . t("from drupal.org") . "</a>",
    )) . "</li>";
  $html .= "</ul>";
  return $html; 
}

/**
 * Get HTML to express the design rationale for Garmin Connector.
 *
 * @return string
 *   HTML expressing design rationale
 */
function garmin_connector_get_design_rationale() {
  $overview = "";
  $overview .= "<h4>" . t("Design Rationale") . "</h4>";
  $overview .= "<p>" . t("I began training for !imcda 2013 in October 2012. As part of my training I decided to create a mechanism to motivate myself to train by gathering necessary information into a !Drupal-based website. I quickly realized that I needed to integrate actual workout data into my website.  Like many athletes, I use !Garmin devices to monitor speed, pace, power, and other activity metrics. Garmin uses a device agent to upload activity data into their website !garminconnect. It's possible to use other services like !trainingpeaks but the middleware for the other services made it difficult to retrieve data and the use of a second set of device agents was irritating. For those reasons I decided to get activity data from !garminconnect.",
    array(
      '!imcda'          => "<a href='http://www.ironman.com/triathlon/events/americas/ironman/coeur-dalene.aspx'>" . t("Ironman Coeur D'Alene") . "</a>",
      '!Garmin'         => "<a href='http://garmin.com'>Garmin</a>",
      '!Drupal'         => "<a href='http://drupal.org'>Drupal</a>",
      '!garminconnect'  => "<a href='http://connect.garmin.com'>" . t("Garmin Connect") . "</a>",
      '!trainingpeaks'  => "<a href='http://trainingpeaks.com'>Training Peaks</a>",
    )
  ) . "</p>";
  $overview .= "<p>" . t("As training progressed I began to meet with my coach and it became nessary to quickly compare two or more workouts. It was at that time that the concept of '!as' was considered.",
  array(
    '!as' => "<a href='" . url("garmin-connector/activity-sets.html") . "'>" .
    t("Activity Sets") . "</a>",
  )
  ) . "</p>";
  $overview .= "<p>" . t("My training also involved doing repetitive tasks like core training and strength workouts. I needed to account for these activities in Garmin Connect but because I didn't use a device to record data I had to manually create activities in Garmin Connect. This was irritating and so I automated the upload of routine activities.") . "</p>";
  $overview .= "<p>" . t("Activities in Garmin Connect are generally fluid for an athlete. Activities are created, deleted, renamed, and annotated. This means that one cannot simply cache activities - a mechanism to maintain the cached activities became necessary to provide up-to-date information.") . "</p>";
  return $overview;
}

/**
 * Create a row to be shown in the Garmin Connector Activity Manager.
 *
 * @param object $activity
 *   A Garmin-Connect style object containing an activity
 *
 * @return array
 *   A drupal render array.
 */
function garmin_connector_get_activity_manager_row($activity) {
  $type = garmin_connector_get_type($activity);
  $name = $activity->activityName;
  $time = new DateTime($activity->activitySummary->BeginTimestamp->value);
  $time->setTimeZone(new DateTimeZone($activity->activitySummary->BeginTimestamp->uom));
  $better_timestamp = $time->format('Y-m-d H:i:s');
  if (property_exists($activity->activitySummary, "SumElapsedDuration")) {
    $duration = $activity->activitySummary->SumElapsedDuration->display;
  }
  if (!isset($duration)) {
    // This happens for my corrective exercises - an..
    // Activity of type other with specified duration.
    $duration = $activity->activitySummary->SumDuration->display;
  }
  $row = array(
    '#theme'    => 'garmin_connector_activity_row',
    '#date'     => $better_timestamp,
    '#duration' => $duration,
    '#name'     => $name,
    '#gc-id'    => $activity->activityId,
    '#classes'  => array(
      'gc-activity-manager-row',
      'gc-activity-manager-row-' . $activity->activityId,
    ),
  );
  $renderable = array(
    '#children' => array(
      '1' => $row,
    ),
    '#theme'    => 'garmin_connector_contextual_links_wrapper',
    '#gc-id'    => $activity->activityId,
    '#gc-type'  => $type,
    '#gc-name'  => $name,
    '#gc-classes' => array(),
    '#gc-css-id' => 'activity-row-' . $activity->activityId,
  );
  return $renderable;
}

/**
 * Show a workout set.
 */
function garmin_connector_show_workout_set($form, &$form_state) {
  if (user_access('view activity sets') == FALSE) {
    $message = "<div class='gc-no-permission'>" . t('Unfortunately, you do not have permission to view activity sets. Ask your system administrator to help!') . "</div>";
    $form_state['csm'] = array('#markup' => $message);
    $form_state['rebuild'] = TRUE;
    return;
  }
  if (user_access('view activities') == FALSE) {
    $message = "<div class='gc-no-permission'>" . t('Although you have permission to view workout sets, you do not have permission to view activity sets. Ask your system administrator to help!') . "</div>";
    $form_state['csm'] = array('#markup' => $message);
    $form_state['rebuild'] = TRUE;
    return;
  }
  garmin_connector_add_css_js();
  $set = $form_state['values']['sets'];
  $returner["0-title"] = array(
    '#markup' => "<h3>" .
    t("Workout set with name @set",
      array('@set' => $set,
      )
    ) . "</h3>",
  );
  $workouts_in_set = garmin_connector_get_workout_set($set);
  $count = 1;
  $returner = array();
  foreach ($workouts_in_set as $id => $row) {
    $type = $row['gc_type'];
    $name = $row['gc_name'];
    $returner[$count . "-top"] = array(
      '#theme'      => 'garmin_connector_contextual_links_wrapper',
      '#gc-id'      => $id,
      '#gc-type'    => $type,
      '#gc-name'    => $name,
      '#gc-classes' => array(
        "rendered-workout-$id",
      ),
      '#gc-css-id' => "activity-set-$set-member-$id",
      '#children' => array(
        '1' => array(
          '#theme'  => 'garmin_connector_plot_container',
          '#id'     => $id,
          '#serial' => $count,
        ),
      ),
    );
    $count++;
  }
  $form_state['csm'] = $returner;
  $form_state['rebuild'] = TRUE;
}

/**
 * Page callback: Constructs a form for choosing a workout set.
 *
 * @see garmin_connector_menu()
 */
function garmin_connector_activity_set_form($form, &$form_state) {
  if (user_access('view activity sets') == FALSE) {
    return array(
      '#suffix' => "<div class='gc-no-permission'>" . t('Unfortunately, you do not have permission to view activity sets. Ask your system administrator to help!') . "</div>",
    );
  }
  garmin_connector_add_css_js();
  if (array_key_exists('csm', $form_state)) {
    $form['csm'] = $form_state['csm'];
    return $form;
  }
  else {
    // Get the workout set form.
    $form = garmin_connector_get_workout_set_base(NULL, FALSE);
    if (!array_key_exists('no-sets-were-found', $form)) {
      $form['submit'] = array(
        '#type'   => 'submit',
        '#value'  => t("Show this workout set"),
        '#submit' => array('garmin_connector_show_workout_set'),
      );
      $form['#submit'] = array('garmin_connector_show_workout_set');
    }
    return $form;
  }
}

/**
 * Page callback: Provide a list of activities.
 *
 * This is done so the user can choose workouts to purge from the
 * filesystem.
 *
 * @see garmin_connector_menu()
 */
function garmin_connector_purge_activities_form($form, &$form_state) {
  module_load_include('inc', 'garmin_connector', 'garmin_connector.catalogue');
  $workouts_found = "";
  try {
    $workouts = garmin_connector_get_activities_from_local(TRUE);
    $workouts_found = "yes";
  }
  catch (Exception $e) {
    $workouts_found = "no";
  }
  if ($workouts_found == "yes") {
    $checkbox_options = array();
    $form['submit2'] = array(
      '#type' => 'submit',
      '#value' => t('Purge Activities from the last month titled "Untitled"'),
      '#suffix' => "<p>" . t("When new activities are created automatically through the Garmin Ant Agent they are given the title 'Untitled'. It's possible that garmin_connector_retrieve_activities ran before you were able to go to Garmin Connect to give the workout a name. This button gives you an easy way to purge those so that when Activities are retrieved again they'll have the correct title") .
      "</p>",
      '#submit' => array('garmin_connector_trigger_purge_untitled'),
    );
    $form['submit'] = array(
      '#type'   => 'submit',
      '#value'   => t('Purge The Activities checked below'),
      '#submit' => array('garmin_connector_process_purge'),
    );
    foreach ($workouts as $workout) {
      $type = garmin_connector_get_type($workout);
      $name = $workout->activityName;
      $time = new DateTime($workout->activitySummary->BeginTimestamp->value);
      $time->setTimezone(new DateTimeZone(date_default_timezone_get()));
      $better_timestamp = $time->format('Y-m-d H:i:s');
      $checkbox_options[$workout->activityId] = "$better_timestamp ($type)
duration(" . $workout->activitySummary->SumDuration->display . "): $name";
    }
    $form['workouts'] = array(
      '#type'   => 'checkboxes',
      '#options'  => $checkbox_options,
      '#title'  => t("Workout List"),
      '#description'  => t("Choose one or more workouts from this list"),
    );
  }
  else {
    $form['workouts'] = array(
      // The exception already t()'d the message.
      '#markup' => "<p><b>" . $e->getMessage() . "</b></p>",
    );
  }
  return $form;
}

/**
 * Create one quick view pane for each of the workouts that the user submitted.
 */
function garmin_connector_get_compare_results($form, &$form_state) {
  if (user_access('view activities') == FALSE) {
    $message = "<div class='gc-no-permission'>" . t('You do not have permission to view activity data so I cannot show you the activity comparison. Ask your system administrator to help!') . "</div>";
    $form_state['csm'] = array('#markup' => $message);
    $form_state['rebuild'] = TRUE;
    return;
  }
  $selected = array();
  foreach ($form_state['values']['workouts'] as $key => $value) {
    if ($value != 0) {
      // There's a small chance of collisions here but this could only be the..
      // Result of evil anyway.
      $selected[check_plain($key)] = 1;
    }
  }
  $returner = array();
  $activities = garmin_connector_get_activities_from_local();
  foreach ($selected as $id => $value) {
    $this_activity = $activities->$id;
    $type = garmin_connector_get_type($this_activity);
    $outer = array(
      '#theme'  => 'garmin_connector_plot_container',
      '#id'     => $id,
      '#serial' => $id,
    );
    // It would be fin to not render this here. Make this a @todo.
    $container = drupal_render($outer);
    $renderable = array(
      '#markup'   => $container,
      '#theme'    => 'garmin_connector_contextual_links_wrapper',
      '#gc-id'    => $id,
      '#gc-type'  => $type,
      '#gc-name'  => check_plain($this_activity->activityName),
      '#gc-classes' => array(),
      '#gc-css-id' => 'compare-results-' . $id,
    );
    $returner[$id] = $renderable;
  }
  $form_state['csm'] = $returner;
  $form_state['rebuild'] = TRUE;
}

/**
 * Page callback: Constructs a form with a list of activities in the catalogue.
 *
 * @see garmin_connector_menu()
 */
function garmin_connector_compare_activities_form($form, &$form_state) {
  module_load_include('inc', 'garmin_connector', 'garmin_connector.catalogue');
  garmin_connector_add_css_js();
  if (array_key_exists('csm', $form_state)) {
    $form['csm'] = $form_state['csm'];
    return $form;
  }
  $workouts = garmin_connector_get_activities_from_local();
  $checkbox_options = array();
  // Sanity checks are done prior to variable storage but let's make sure.
  $ignore_regex = variable_get('garmin_connector_key_workout_block_ignore_regex');
  foreach ($workouts as $workout) {
    $type = garmin_connector_get_type($workout);
    $name = $workout->activityName;
    if (strlen($ignore_regex) > 0) {
      $matches = preg_match("/$ignore_regex/i", $name);
      if ($matches > 0) {
        continue;
      }
    }
    $time = new DateTime($workout->activitySummary->BeginTimestamp->value);
    $better_timestamp = $time->format('Y-m-d H:i:s');
    $checkbox_options[$workout->activityId] = "$better_timestamp ($type) : $name";
  }
  $form['workouts'] = array(
    '#type'   => 'checkboxes',
    '#options'  => $checkbox_options,
    '#title'  => t("Workout List"),
    '#description'  => t("Choose one or more workouts from this list"),
  );
  $form['submit'] = array(
    '#type'   => 'submit',
    '#value'   => t('Do it!'),
    '#submit' => array('garmin_connector_get_compare_results'),
  );
  return $form;
}

/**
 * Get an html list of the drush commands that Garmin Connector provides.
 *
 * @return array
 *   A drupal render array
 */
function garmin_connector_get_drush_commands() {
  $stringer = "<div class='gc-drush-commands clearfix'>";
  module_load_include('inc', 'garmin_connector', 'garmin_connector.drush');
  $commands = garmin_connector_drush_command();
  foreach ($commands as $command => $definition) {
    $stringer .= "<div class='gc-drush-command'>";
    $stringer .= "<div class='name'>";
    $stringer .= $command;
    $stringer .= "</div>";
    $stringer .= "<div class='description'>";
    $stringer .= $definition['description'];
    $stringer .= "</div>";
    $stringer .= "</div>";
  }
  $stringer .= "</div>";
  return array(
    '#markup' => $stringer,
  );
}

/**
 * Validate the form from garmin_connector_admin.
 */
function garmin_connector_admin_validate($form, &$form_state) {
  // Make all of these values plain.
  if (strlen($form_state['values']['garmin_connector_pw']) == 0) {
    $form_state['values']['garmin_connector_pw']
      = variable_get('garmin_connector_pw');
  }
}

/**
 * Implements hook_garmin_connector_strength_workout_types().
 */
function garmin_connector_strength_workout_types() {
  return array(
    'A hard weights workout' => 'A hard weights workout',
  );
}

/**
 * Get the color key to display in various places.
 *
 * Turned into a method for consistency
 *
 * @return array
 *   A drupal render array
 */
function garmin_connector_colors_get_key() {
  $returner  = "<div class='color-key'>";
  $returner .= "<div class='sport swim'>" . t("Swim") . "</div>";
  $returner .= "<div class='sport bike'>" . t("Bike") . "</div>";
  $returner .= "<div class='sport run'>" . t("Run") . "</div>";
  $returner .= "<div class='sport other'>" . t("Other") . "</div>";
  $returner .= "<div class='sport strength'>" . t("Strength") . "</div>";
  $returner .= "<div class='sport dayoff'>" . t("Day Off") . "</div>";
  $returner .= "</div>";
  return array('#markup' => $returner);
}

/**
 * Get the contextual links.
 *
 * @param int $id
 *   GC ID
 *
 * @param string $type
 *   GC type
 *
 * @param string $name
 *   The GC name of the workout
 * 
 * @return array
 *   A drupal render array
 */
function garmin_connector_get_contextual_links($id, $type, $name) {
  // The problem here is that if you try to replace a activity-row with..
  // A new one (because of a refresh) the forms are stale and cannot be added..
  // To the new contectual links. This is why I need to ask for information...
  // From ajax_get_form. If that information is absent I can simply build the...
  // Form.
  $form_id = "unknown";
  if (array_key_exists('form_build_id', $_POST)) {
    list($form, $form_state, $form_id, $form_build_id) = ajax_get_form();
  }
  if (isset($form_build_id)) {
    // This is a false positive with pareview.sh.
  }
  $plot_uri = "public://garmin_connector/activities/plot-data/$id.json";
  $plot_url = file_create_url($plot_uri);
  $returner = "";
  $returner .= "<ul class='contextual-links'>\n";
  $returner .= "<li class='purge'>\n";
  $returner .= "<div class='purge-$id purge-activities-form-wrapper'>\n";
  $newform1 = array();
  if ($form_id == "garmin_connector_purge_activities_form") {
    $newform1 = drupal_build_form($form_id, $form_state);
  }
  else {
    $newform1 = drupal_get_form("garmin_connector_purge_activities_ajax_form", $id);
  }
  $returner .= drupal_render($newform1);
  $returner .= "</div><!-- /purge-$id -->\n";
  $returner .= "</li>\n";
  $returner .= "<li class='refresh'>\n";
  $returner .= "<div class='refresh-$id refresh-activities-form-wrapper'>\n";
  $newform2 = array();
  if ($form_id == "garmin_connector_refresh_activities_ajax_form") {
    $newform2 = drupal_build_form($form_id, $form_state);
  }
  else {
    $newform2 = drupal_get_form("garmin_connector_refresh_activities_ajax_form", $id);
  }
  $returner .= drupal_render($newform2);
  $returner .= "</div><!-- /refresh-$id -->\n";
  $returner .= "</li>\n";
  $returner .= "<li><a href='$plot_url'>" . t("View raw plot data");
  $returner .= "</a></li>\n";
  $returner .= "<li><a href='http://connect.garmin.com/activities/$id'>";
  $returner .= t("View in Garmin Connect") . "</a></li>\n";
  $returner .= "<li>";
  $returner .= t("Add to workout set:") . "<ul class='maintain-activity-set'>\n";
  $returner .= "<li>\n";
  $form = drupal_get_form("garmin_connector_add_to_activity_set_form",
    $id, $name, $type);
  $returner .= drupal_render($form);
  $returner .= "</li>";
  $form2 = drupal_get_form("garmin_connector_remove_from_activity_set_form",
    $id, $type);
  $returner .= "<li>" . drupal_render($form2);
  $returner .= "<div id='activity-set-remove-state-$id'></div>";
  $returner .= "</li>\n";
  $returner .= "</ul><!-- /ul-maintain-activity-sets -->\n";
  $returner .= "</li>\n";
  $returner .= "</ul><!-- /contextual-links -->\n";
  return $returner;
}

/**
 * Get a form for adding a workout to a workout set.
 * 
 * @param int $workoutid
 *   The workout ID of this workout
 *
 * @returns array
 *   A form
 */
function garmin_connector_get_workout_set_base($workoutid) {
  module_load_include('inc', 'garmin_connector', 'garmin_connector.catalogue');
  // Get a list of workout sets for this user.
  $sets = garmin_connector_get_workout_set_names($workoutid, TRUE);
  $form = array();
  $form['#attributes'] = array(
    'class' => array(
      "add-to-activity-set-form",
      "add-to-activity-set-form-$workoutid"),
  );
  if ($sets) {
    $form['sets'] = array(
      '#type'   => 'select',
      '#options' => $sets,
      '#default_value'  => '',
    );
  }
  else {
    $form['no-sets-were-found'] = array(
      '#markup' => "<p>" . t("No existing workout sets were found. You can create a workout set in the Garmin Connector activity manager !here",
      array(
        '!here' => "<a href='" . url('garmin-connector/activity-manager.html') .
        "'>" . t("here") . "</a>",
      )
      ) . "</p>",
    );
  }
  return $form;
}

/**
 * Ajax callback for running garmin_connector_retrieve_activities.
 *
 * Any Exceptions are nicely reported back to the element that summoned
 * us.
 */
function garmin_connector_ajax_retrieve_activities($form, &$form_state) {
  module_load_include('inc', 'garmin_connector', 'garmin_connector.catalogue');
  $message = "";
  try {
    $message = t("Everything is ok. Reload to see new activities.");
    garmin_connector_retrieve_activities();
  }
  catch(Exception $e) {
    $message = t("retrieve_activities failed with the following message: !error",
      array(
        '!error' => $e->getMessage(),
      )
    );
  }
  return array(
    '#type' => 'ajax',
    '#commands' => array(
      ajax_command_replace(".retrieve-activities-results",
      "<div class='.retrieve-activities-results'>" .
      $message . "</div>"),
    ),
  );
}

/**
 * Refresh a single activity.
 *
 * This method is intended to be called from an ajax button.
 *
 * @see garmin_connector_refresh_activity()
 *
 * @todo
 * This replaces an activity graph with an activity row. Determine when a
 * graph is needed and output that.
 */
function garmin_connector_ajax_refresh_activity($form, &$form_state) {
  module_load_include('inc', 'garmin_connector', 'garmin_connector.catalogue');
  $id = preg_replace("/activity-row-/", "", check_plain($form_state['values']['garmin-id']));
  if (!$id) {
    return t("Id @id was not found in the garmin connector catalogue",
      array(
        '@id' => $id,
      )
    );
  }
  $status  = "<div class='refresh-activity-results'>";
  try {
    garmin_connector_refresh_activity($id);
    $cat = garmin_connector_get_activities_from_local();
    $by_id = garmin_connector_key_by_id($cat);
    $new_row = garmin_connector_get_activity_manager_row($by_id[$id]);
    $status .= "<p>";
    $status .= t("Everything succeeded in refreshing an activity with id @id",
      array(
        '@id' => $id,
      )
    );
    $status .= "</p>";
    $message = drupal_render($new_row);
  }
  catch (Exception $e) {
    $status  .= "<b>";
    $status .= $e->getmessage();
    $status .= "</b>";
    $message = "";
  }
  // The catalogue will have been refreshed.
  return array(
    '#type' => 'ajax',
    '#commands' => array(
      ajax_command_replace("#activity-row-$id", $status . $message . "</div>"),
      ajax_command_replace("#last-ten-$id", $status . $message . "</div>"),
    ),
  );
}

/**
 * Purge a single activity.
 *
 * @return string
 *   An ajax array.
 */
function garmin_connector_purge_single($form, &$form_state) {
  module_load_include('inc', 'garmin_connector', 'garmin_connector.catalogue');
  $cat = garmin_connector_get_activities_from_local();
  $by_id = garmin_connector_key_by_id($cat);
  $inid = preg_replace("/activity-row-/",
     "",
     check_plain($form_state['values']['garmin-id']));
  if (!array_key_exists($inid, $by_id)) {
    return t("Id @inid was not found in the garmin connector catalogue",
      array(
        '@inid' => $inid,
      )
    );
  }
  $activities = array($inid);
  $locations = garmin_connector_purge_activities($activities);
  $returner = "<div class='purge-results purge-results-" . $inid;
  $returner .= " purge-activities-form-wrapper'>";
  foreach ($locations as $location => $status) {
    $returner .= "<p>" . t("Purging !location status: !status",
      array(
        '!location' => $location,
        '!status'   => $status,
      )
    ) . "</p>";
  }
  $returner .= "</div>";
  return array(
    '#type' => 'ajax',
    '#commands' => array(
      ajax_command_replace(
        "#activity-row-$inid",
        $returner
      ),
      ajax_command_replace(
        "#last-ten-$inid",
        $returner
      ),
    ),
  );
}

/**
 * Get a form to download sample data.
 *
 * @return array
 *   An AJAX array
 */
function garmin_connector_ajax_get_sample_data_form($form, &$form_state) {
  $form['submit'] = array(
    '#type'   => 'submit',
    '#value'  => t("Download sample data"),
    '#ajax'   => array(
      'callback'  => 'garmin_connector_ajax_get_sample_data',
      'keypress'  => TRUE,
      'prevent'   => 'click',
      'progress'  => array(
        'type'    => 'throbber',
        'message' => t("Downloading sample data (could take a bit)..."),
      ),
    ),
  );
  return $form;
}

/**
 * Ajax callback for downloading sample data.
 */
function garmin_connector_ajax_get_sample_data() {
  $message = "All done!";
  try {
    garmin_connector_install_sample_data();
    $message = t("Sample data downloaded");
  }
  catch(Exception $e) {
    $message = t("Downloading sample data failed because !error",
    array(
      '!error' => $e->getMessage(),
    )
    );
  }
  return array(
    '#type' => 'ajax',
    '#commands' => array(
      ajax_command_replace(".sample-done-results",
      "<div class='.sample-done-results'>" .
      $message . "</div>"),
    ),
  );
}

/**
 * Get a Retrieve Activities Now form.
 *
 * @return array
 *   An AJAX array
 */
function garmin_connector_ajax_run_retrieve_activities_form($form, &$form_state) {
  $form['submit'] = array(
    '#type'   => 'submit',
    '#value'  => t("Retreieve Activities Now"),
    '#ajax'   => array(
      'callback'  => 'garmin_connector_ajax_retrieve_activities',
      'keypress'  => TRUE,
      'prevent'   => 'click',
      'progress'  => array(
        'type'    => 'throbber',
        'message' => t("Retrieving activities from Garmin Connect (could take a bit)..."),
      ),
    ),
    '#suffix' => '<div class="retrieve-activities-results"></div>',
  );
  return $form;
}

/**
 * Get an exercises done form.
 *
 * @return array
 *   An AJAX array
 */
function garmin_connector_ajax_exercises_done_form($form, &$form_state) {
  $form['submit'] = array(
    '#type'   => 'submit',
    '#value'  => t("I completed my exercises!"),
    '#ajax'   => array(
      'callback'  => 'garmin_connector_exercises_done',
      'keypress'  => TRUE,
      'prevent'   => 'click',
      'progress'  => array(
        'type'    => 'throbber',
        'message' => t("Creating..."),
      ),
    ),
  );
  $form['#suffix'] = "<div class='exercises-done-results'></div>";
  return $form;
}

/**
 * Get a form for indicating that strength exercises are done.
 */
function garmin_connector_ajax_strength_done_form($form, &$form_state) {
  $form['#title'] = t("Add a new strength workout");
  $form['#attributes'] = array(
    'class'   => array('strength-done-form'),
  );
  $form['#suffix'] = "<div class='strength-done-results'></div>";
  $form['duration'] = array(
    '#title'    => t('Duration'),
    '#type'     => 'select',
    '#options'  => array(
      40 => 40, 45 => 45, 50 => 50, 55 => 55, 60 => 60,
      65 => 65, 70 => 70, 75 => 75,
      80 => 80, 85 => 85, 90 => 90),
    '#empty_option' => t('Pick One'),
    '#required'    => TRUE,
  );
  $form['type'] = array(
    '#title'    => t('Type'),
    '#type'     => 'select',
    '#options'  => module_invoke_all('strength_workout_types'),
    '#empty_option' => 'Pick One',
    '#required'    => TRUE,
  );
  $form['submit'] = array(
    '#type'   => 'submit',
    '#value'  => t("I completed my strength workout"),
    '#ajax'   => array(
      'callback'  => 'garmin_connector_strength_done',
      'keypress'  => TRUE,
      'prevent'   => 'click',
      'progress'  => array(
        'type'    => 'throbber',
        'message' => t("Creating strength exercise..."),
      ),
    ),
  );
  return $form;
}

/**
 * Get a form to refresh an activity.
 *
 * @param int $activityid
 *   The ActivityId of the activity for which this form is being created.
 */
function garmin_connector_refresh_activities_ajax_form($form, &$form_state, $activityid) {
  $form = array();
  if (!user_access("manage activities") == TRUE) {
    return array();
  }
  $form['garmin-id'] = array(
    '#type'    => 'hidden',
    '#value'  => $activityid,
  );
  $form['submit2'] = array(
    '#description' => t("You may have renamed or modified an activity at Garmin Connect. This will re-download activity data from Garmin Connect"),
    '#type'   => 'submit',
    '#value'  => "Refresh this activity",
    '#ajax'   => array(
      'callback'  => 'garmin_connector_ajax_refresh_activity',
      'keypress'  => TRUE,
      'prevent'   => 'click',
      'progress'  => array(
        'type'    => 'throbber',
        'message' => t("Refreshing @id...", array('@id' => $activityid)),
      ),
    ),
    '#suffix' => "<div class='refresh-results-$activityid'></div>",
  );
  return $form;
}

/**
 * Get a form to purge an activity.
 *
 * @param int $activityid
 *   The ActivityId of the activity for which this form is being created.
 */
function garmin_connector_purge_activities_ajax_form($form, &$form_state, $activityid) {
  if (!user_access("manage activities") == TRUE) {
    return array();
  }
  $form = array();
  $form['garmin-id'] = array(
    '#type'    => 'hidden',
    '#value'  => $activityid,
  );
  $form['submit'] = array(
    '#type'   => 'submit',
    '#value'  => "Purge this activity",
    '#ajax'   => array(
      'callback'  => 'garmin_connector_purge_single',
      'keypress'  => TRUE,
      'prevent'   => 'click',
      'progress'  => array(
        'type'    => 'throbber',
        'message' => t("Purging @id...", array('@id' => $activityid)),
      ),
    ),
    '#suffix' => "<div class='purge-results-$activityid'></div>",
  );
  return $form;
}

/**
 * Get a form to add a workout to a activity set.
 */
function garmin_connector_add_to_activity_set_form($form,
    &$form_state, $workoutid = NULL, $workoutname = NULL, $workouttype = NULL) {
  if (user_access("maintain activity sets") == FALSE) {
    return array();
  }
  $form = garmin_connector_get_workout_set_base($workoutid, TRUE);
  // In this case, do not tell the user where they can add a new set. That's.
  // Here!
  $form['#attributes'] = array(
    'class' => array(
      'csm-create-new-set',
    ),
  );
  unset($form['no-sets-were-found']);
  $form['garmin-id'] = array(
    '#type'    => 'hidden',
    '#value'  => $workoutid,
  );
  $form['name'] = array(
    '#type'   => 'hidden',
    '#value'  => $workoutname,
  );
  $form['type'] = array(
    '#type'   => 'hidden',
    '#value'  => $workouttype,
  );
  $form['new'] = array(
    '#type'   => 'textfield',
    '#size'   => '10',
    '#title'  => 'Create New Set',
    '#maxlength' => 30,
  );
  $form['submit'] = array(
    '#type'   => 'submit',
    '#value'  => "Add",
    '#ajax'   => array(
      'callback'  => 'garmin_connector_add_to_workout_set',
      'wrapper'   => 'add-to-activity-set-form',
      'keypress'  => TRUE,
      'prevent'   => 'click',
      'progress'  => array(
        'message' => t('Adding...'),
      ),
    ),
  );
  return $form;
}

/**
 * Returns HTML for a Garmin Connector pseudoblock.
 *
 * Garmin Connector needs to make sure that elements intended to be wrapped in a
 * block have css classes like 'clearfix' added.
 *
 * @param array $block
 *   A block that needs to be wrapped.
 *
 * @ingroup themeable
 */
function theme_garmin_connector_block_wrapper($block) {
  // We don't need the theme information anymore.
  unset($block['element']['#theme']);
  // Now that everything is rendered, we don't need the prefix or suffix..
  // anymore. If you don't remove these they will be added again.
  unset($block['element']['#prefix']);
  unset($block['element']['#suffix']);
  $html = "<div class='csm-block clearfix'>";
  $html .= drupal_render($block['element']);
  $html .= "</div>";
  return $html;
}

/**
 * Returns HTML for a Garmin Connector plot container.
 *
 * Garmin Connector will place a graph into this box.
 *
 * @param array $element
 *   An associative array containing information about the plot container
 *     - count  A serial number for the container
 *     - id     The Garmin Connector Activity Id for the thing in the container
 *
 * @ingroup themeable
 */
function theme_garmin_connector_plot_container($element) {
  // This is a bit goofy but it will be improved as the module evolves.
  // At least it's themeable.
  $renderer = garmin_connector_get_plot_container($element['serial'], $element['id']);
  return drupal_render($renderer);
}

/**
 * Returns HTML for a row containing an activity.
 *
 * @param array $element
 *   An associative array containing information about a manageable activity.
 *     - timestamp The timestamp for the activity.
 *     - duration  The duration of the activity.
 *     - name      The name of the activity.
 *     - classes   Classes to put in the container div.
 *
 * @ingroup themeable
 */
function theme_garmin_connector_activity_row($element) {
  $classes = implode(" ", $element['classes']);
  if (!$classes) {
    $classes = "gc-row";
  }
  $markup  = "<div class='$classes'>";
  // This was span.workout-date in the last-ten.
  $markup .= "<div class='workout-date date'>" . $element['date'] . "</div>\n";
  // This was a span in the last-ten.
  $markup .= "<div class='duration date-duration'>" . $element['duration'] . "</div>\n";
  $markup .= "<div class='name activity-name'>" . $element['name'] . "</div>\n";
  $markup .= "<a class='gc-player' href='https://connect.garmin.com/";
  $markup .= "player/" . $element['gc-id'] . "'>GC Player</a>\n";
  // Hook_theme is supposed to use my values as defaults but is not doing that.
  // For example, when '#children' isn't present, there should be an empty..
  // Array as specified in hook_theme(). This doesn't happen. Why?
  if (is_array($element['children'])) {
    foreach ($element['children'] as $child) {
      $markup .= drupal_render($child);
    }
  }
  $markup .= "</div><!-- /workout-manager row -->\n";
  return $markup;
}
/**
 * Returns HTML for an item that should have Garmin Connector contextual links.
 *
 * Garmin Connector can add contextual links for HTML containing information
 * about an Activity. This can simplify the maintenance of activities in the
 * Garmin Connector Activity Catalogue.
 *
 * @param array $element
 *   An associative array containing information about an activity.
 *     - gc-css-id The intended css id for the wrapped element
 *     - gc-type   The type of the activity
 *     - gc-id     The activity id
 *     - gc-classes  Additional classes that need to be added to the wrapping
 *      element.
 *       - classes already included are: clearfix csm contextual-links-wrapper
 *     - markup    The HTML to be wrapped in contextual links
 *
 * @ingroup themeable
 */
function theme_garmin_connector_contextual_links_wrapper($element) {
  garmin_connector_add_css_js();
  $classes = array(
    $element['gc-type'],
    'clearfix',
  );
  $classes = array_merge($classes, $element['gc-classes']);
  $header = "<div id='" . $element['gc-css-id'] . "' ";
  if (user_access("manage activities")) {
    array_push($classes, "csm", "contextual-links-wrapper");
    $links = garmin_connector_get_contextual_links($element['gc-id'],
      $element['gc-type'],
      $element['gc-name']);
  }
  $header .= " class='" . implode(" ", $classes) . "'>";
  if (isset($links)) {
    // This can't be a render array because of the way it's nested..
    // With the parent element.
    $header .= $links;
  }
  $body = $element['markup'];
  if (!is_array($element['children'])) {
  }
  else {
    foreach ($element['children'] as $child) {
      $container = drupal_render($child);
      $body .= $container;
    }
  }
  return $header . $body . "</div>";
}

/**
 * Add css and js necessary for Garmin Connector's gui components.
 */
function garmin_connector_add_css_js() {
  drupal_add_css(drupal_get_path('module', 'garmin_connector')
    . '/garmin_connector.css');
  drupal_add_js(drupal_get_path('module', 'garmin_connector')
    . '/garmin_connector.js');
}

/**
 * Get a form to allow the user to remove an activity from an activity set.
 *
 * @param int $id
 *   The id of the activity belonging to this form
 *
 * @param string $type
 *   The type of this activity
 * 
 * @return array
 *   A form
 */
function garmin_connector_remove_from_activity_set_form($form, &$form_state, $id, $type) {
  $returner = array();
  $returner['#attributes'] = array(
    'class' => array(
      "remove-from-activity-set-form",
      "remove-from-activity-set-form-$id"),
  );
  $options = garmin_connector_get_my_workout_sets($id);
  $returner['type'] = array(
    '#type' => 'hidden',
    '#value' => $type,
  );
  $returner['garmin-id'] = array(
    '#type' => 'hidden',
    '#value' => $id,
  );
  if (count($options) > 0) {
    $returner['sets'] = array(
      '#type' => 'checkboxes',
      '#options' => $options,
      '#title' => t("Remove from set:"),
    );
    $returner['submit'] = array(
      '#type'   => 'submit',
      '#value'  => t("Remove"),
      '#ajax'   => array(
        'callback'  => 'garmin_connector_remove_from_workout_set',
        'wrapper'   => 'set-remove-done',
        'keypress'  => TRUE,
        'prevent'   => 'click',
      ),
    );
  }
  return $returner;
}

/**
 * Ajax to remove a workout from a workout set.
 * 
 * @return array
 *   An ajax array containing information for the results of the attempt.
 *
 * @todo
 *   Exceptions would be good for handling unexpected conditions
 */
function garmin_connector_remove_from_workout_set($form, &$form_state) {
  if (!user_access("manage activities")) {
    // Throwing a more helpful error is a todo. Exception?
    drupal_set_message(t("Sorry, you don't have the permission to remove an activity from a workout set. You need the permission 'Manage Activities'"));
    return;
  }
  // Suppress unused error. Remove if necessary.
  $id = $form_state['values']['garmin-id'];
  foreach ($form_state['values']['sets'] as $set) {
    $num_deleted = db_delete('garmin_connector_workout_sets')
      ->condition('id_workout_sets', $set)
      ->execute();
    if ($num_deleted == FALSE) {
      // Throw an error. Hard to do, given ajax.
      drupal_set_message(t("An unknown error occurred when trying to remove an activity from a workout set"));
    }
  }
  $add_form = drupal_rebuild_form(
    "garmin_connector_add_to_activity_set_form",
    $form_state);
  $remove_form = drupal_rebuild_form(
    "garmin_connector_remove_from_activity_set_form",
    $form_state);
  return array(
    '#type' => 'ajax',
    '#commands' => array(
      ajax_command_replace(".add-to-activity-set-form-$id",
        drupal_render($add_form)),
      ajax_command_replace(".remove-from-activity-set-form-$id",
        drupal_render($remove_form)),
    ),
  );
}

/**
 * Add a workout to a workout set.
 *
 * @todo
 *   Exceptions to handle database errors would be useful.
 */
function garmin_connector_add_to_workout_set($form, &$form_state) {
  if ($form_state['values']['new'] != '') {
    $set = $form_state['values']['new'];
  }
  else {
    $set = $form_state['values']['sets'];
  }
  if ($set == "None") {
    return;
  }
  $id = $form_state['values']['garmin-id'];
  $name = $form_state['values']['name'];
  $type = $form_state['values']['type'];
  // First, check to see if the workout is already in there.
  $query = db_select('garmin_connector_workout_sets', 's')
    ->addField('s', 'id_workout_sets')
    ->condition('s.set_name', $set, '=')
    ->condition('s.gc_id', $id, '=')
    ->execute();
  $records = $query->rowCount();
  $message = "";
  if ($records == 0) {
    global $user;
    $select = db_insert("garmin_connector_workout_sets")
      ->fields(array(
          'user'  => $user->name,
          'set_name' => $set,
          'gc_id'    => $id,
          'gc_name'  => $name,
          'gc_type'  => $type,));
    $select->execute();
    $message .= "Added to workout set $set";
  }
  else {
    $message .= "Already in set $set.";
  }
  $add_form = drupal_rebuild_form(
    "garmin_connector_add_to_activity_set_form",
    $form_state);
  $remove_form = drupal_rebuild_form(
    "garmin_connector_remove_from_activity_set_form",
    $form_state);
  return array(
    '#type' => 'ajax',
    '#commands' => array(
      ajax_command_replace(".add-to-activity-set-form-$id",
      drupal_render($add_form)),
      ajax_command_replace(".remove-from-activity-set-form-$id",
      drupal_render($remove_form)),
    ),
  );
}

/**
 * Look into the form_state and get the list of checked boxes.
 *
 * Pass the IDs to the next stage.
 */
function garmin_connector_process_purge($form, &$form_state) {
  $checked = array();
  foreach ($form_state['values']['workouts'] as $key => $value) {
    if (!$value) {
      continue;
    }
    $fixed = preg_replace("/activity-row-/", "", check_plain($key));
    array_push($checked, $fixed);
  }
  $ids = garmin_connector_prepare_purge($checked);
  $returner = garmin_connector_purge_activities($ids);
  return $returner;
}

/**
 * Get links for the demo page.
 *
 * I do this because if the user has no permissions, the demo page is
 * essentially blank. The Drupal common practice is to not provide a link to a
 * page for which the user has no permission.
 *
 * Since this is a brand new install, there is a reasonable likelihood that
 * this will be the case.
 * 
 * @param string $permission
 *   The permission for the link.
 *
 * @param string $link
 *   The url of the link (do not format with url())
 *
 * @param string $text
 *   NOTE: This string should already be translated with t().
 *
 * @return string
 *   An HTML formatted string.
 */
function garmin_connector_get_demo_link($permission, $link, $text) {
  $returner = "";
  if (user_access($permission) === TRUE) {
    $returner .= "<a href='" . url($link) . "'>$text</a>";
  }
  else {
    $returner .= $text . " (You do not have permission to see this. You either need to log in or get help from your administrator)";
  }
  return $returner;
}
